{------------------------------------------------------------------------------}
{ TWAIN Sample Source                                                          }
{ Copyright (C) '91-'93 TWAIN Working Group:                                   }
{ Aldus, Caere, Eastman-Kodak, Logitech,                                       }
{ Hewlett-Packard Corporations.                                                }
{ All rights reserved.                                                         }
{                                                                              }
{ This module contains routines to manipulate containers.  Containers are      }
{ used in CAPabilty negotiations to exchange information about the input       }
{ device between the Source and the App.  The information may define data      }
{ exchange or input device characteristics.                                    }
{                                                                              }
{                                                                              }
{ This source code was converted from C to Delphi/Turbo Pascal for Windoes by  }
{                                                                              }
{   MCM Design                                                                 }
{   Marc Christopher Martin                                                    }
{   Australiensvej 34                                                          }
{   2100 Copenhagen OE                                                         }
{   Denmark                                                                    }
{                                                                              }
{   e-mail support@mcm-design.dk                                               }
{                                                                              }
{------------------------------------------------------------------------------}

unit twd_com;

{------------------------------------------------------------------------------}
{$A+} { Align data Byte = $A-, Word = $A+.                                     }
{$F+} { Force FAR call.                                                        }
{$G+} { Generate 80286 code.                                                   }
{$N-} { CoProcessor code generation.                                           }
{$R-} { Range check.                                                           }
{$W-} { Protected mode only $W-, Real mode stack frame support $W+.            }
{$X+} { Extended syntax (When on, function can be used as procedures).         }
{------------------------------------------------------------------------------}

interface

uses Twain;

procedure ExtractEnumerationIndex(    pData    : pTW_CAPABILITY;
                                  var Index    : integer);

procedure ExtractEnumerationValue(pData    : pTW_CAPABILITY;
                                  pVoid    : pointer;
                                  j        : integer);

procedure BuildUpEnumerationType( pData    : pTW_CAPABILITY;
                                  pE       : pTW_Enumeration;
                                  pList    : pointer;
                                  Msg      : TW_UINT16);


procedure BuildUpArrayType   (    pData    : pTW_CAPABILITY;
                                  pA       : pTW_ARRAY;
                                  pList    : pointer;
                                  Msg      : TW_UINT16);

procedure BuildUpRangeType   (    pData    : pTW_CAPABILITY;
                                  pR       : pTW_RANGE;
                                  Msg      : TW_UINT16);

function  BuildUpOneValue    (    pData    : pTW_CAPABILITY;
                                  ItemType : TW_UINT16;
                                  Item     : pointer;
                                  Msg      : TW_UINT16) : TW_UINT16;

procedure ExtractOneValue    (    pData    : pTW_CAPABILITY;
                                  pVoid    : pointer);

function  AltTWItemSize      (    ItemType : TW_INT16) : TW_UINT16;


implementation

uses WinTypes, WinProcs, SysUtils;

procedure ExtractEnumerationIndex(    pData    : pTW_CAPABILITY;
                                  var Index    : integer);
var pEnumeration : pTW_ENUMERATION;
begin
  pEnumeration := pTW_ENUMERATION(GlobalLock(pData^.hContainer));
  if (pEnumeration <> Nil)
  then begin
       Index := pEnumeration^.CurrentIndex;
       GlobalUnlock(pData^.hContainer);
  end;
  { free is by the App.                                                        }
end; { End ExtractEnumerationIndex.                                            }


{------------------------------------------------------------------------------}
{ FUNCTION: ExtractEnumerationValue                                            }
{                                                                              }
{ ARGS:    pData   pointer to a capability structure, details about container  }
{          pVoid   ptr will be set to point to the item on return              }
{          j       requested index into the enumeration                        }
{                                                                              }
{ RETURNS: pVoid   is set to pointer to itemtype                               }
{                                                                              }
{ NOTES:   This routine will open a container and extract the Item.  The       }
{ Item will be returned to the caller in pVoid.  I will type cast the returned }
{ value to that of ItemType.                                                   }
{                                                                              }
{ COMMENTS: only a single value is returned.  It is referred to by the 'j'     }
{ index.                                                                       }
{                                                                              }
{ BACKGROUND:  Protocol: used by MSG_SET calls were Source empties then App    }
{ frees upon return.  It is assumed that the APP allocates and fills the       }
{ container prior to this call.                                                }
{------------------------------------------------------------------------------}

procedure ExtractEnumerationValue(pData : pTW_CAPABILITY;
                                  pVoid : Pointer;
                                  j     : integer);
// j = index in Enumeration.
var pEnumeration : pTW_ENUMERATION;
    pItemList    : pointer;
begin
  pEnumeration := pTW_ENUMERATION(GlobalLock(pData^.hContainer));
  if (pEnumeration <> Nil)
  then begin
       { assign base address of ItemList array to 'generic' pointer.           }
       pItemList := @pEnumeration^.ItemList;

       { CAST to type of var caller wants.                                     }
       case pEnumeration^.ItemType of
       TWTY_INT8   : TW_INT8(pVoid^)   := TW_AINT8(pItemList^)[j];
       TWTY_UINT8  : TW_UINT8(pVoid^)  := TW_AUINT8(pItemList^)[j];
       TWTY_INT16  : TW_INT16(pVoid^)  := TW_AINT16(pItemList^)[j];
       TWTY_UINT16 : TW_UINT16(pVoid^) := TW_AUINT16(pItemList^)[j];
       TWTY_INT32  : TW_INT32(pVoid^)  := TW_AINT32(pItemList^)[j];
       TWTY_UINT32 : TW_UINT32(pVoid^) := TW_AUINT32(pItemList^)[j];
       TWTY_BOOL   : TW_BOOL(pVoid^)   := TW_ABOOL(pItemList^)[j];
       TWTY_FIX32  : TW_FIX32(pVoid^)  := TW_FIX32(TW_AFIX32(pItemList^)[j]);
{ $IFDEF DEBUG}
       TWTY_FRAME  : TW_FRAME(pVoid^)  := TW_FRAME(TW_AFRAME(pItemList^)[j]);
       TWTY_STR32  : lstrcpy(TW_STR32(pVoid^),  TW_ASTR32(pItemList^)[j]);
       TWTY_STR64  : lstrcpy(TW_STR64(pVoid^),  TW_ASTR64(pItemList^)[j]);
       TWTY_STR128 : lstrcpy(TW_STR128(pVoid^), TW_ASTR128(pItemList^)[j]);
       TWTY_STR255 : lstrcpy(TW_STR255(pVoid^), TW_ASTR255(pItemList^)[j]);
{ $ENDIF}
       else          begin
	             end;
       end;
       GlobalUnlock(pData^.hContainer);
  end;
  { free is by the App.                                                        }
end; { End ExtractEnumerationValue.                                            }


{------------------------------------------------------------------------------}
{ FUNCTION: BuildUpEnumerationType                                             }
{                                                                              }
{ ARGS:    pData   pointer to a capability structure, details about container  }
{          pE      ptr to struct that contains the other fields of ENUM struct }
{          *pList  ptr to array of elements to put into the ENUM array         }
{                                                                              }
{ RETURNS: pData^.hContainer set to address of the container handle, ptr is    }
{          returned here                                                       }
{                                                                              }
{ NOTES:   The routine dynamically allocates a chunk of memory large enough    }
{ to contain all the struct pTW_ENUMERATION as well as store it's ItemList     }
{ array INTERNAL to the struct.  The array itself and it's elements must be    }
{ type cast to ItemType.  I do not know how to dynamically cast elements       }
{ of an array to ItemType so it is time for a big static switch.>>>            }
{                                                                              }
{ Protocol: Used by MSG_GET.. calls were Source allocates the container and    }
{ the APP uses and then frees the container.                                   }
{                                                                              }
{------------------------------------------------------------------------------}

procedure BuildUpEnumerationType(pData : pTW_CAPABILITY;
                                 pE    : pTW_ENUMERATION;
                                 pList : pointer;         { void *pList       }
                                 Msg   : TW_UINT16);
var pEnumeration : pTW_ENUMERATION; { template for ENUM fields                 }
    j            : integer;         { anyone with more than 32K array elements }
                                    { should crash.  Could type on NumItems.   }
    pVoid        : pointer;
begin
  if (Msg <> MSG_SET)
  then begin
       { Allocate a block large enough for struct and complete enumeration      }
       { array.                                                                 }
       pData^.hContainer := GlobalAlloc(GMEM_MOVEABLE, SizeOf(TW_ENUMERATION) -
                                                       SizeOf(TW_UINT8) +
		                                       pE^.NumItems *
                                                       AltTWItemSize(pE^.ItemType));
  end;
  if (pData^.hContainer <> 0)
  then begin
       { fill in contype.                                                      }
       pData^.ConType := TWON_ENUMERATION;
       pEnumeration   := pTW_ENUMERATION(GlobalLock(pData^.hContainer));
       if (pEnumeration <> Nil)
       then begin
            pEnumeration^.ItemType     := pE^.ItemType;     { TWTY_XXXX        }
            pEnumeration^.NumItems     := pE^.NumItems;     { TWPT_XXXX...     }
            pEnumeration^.CurrentIndex := pE^.CurrentIndex; { current index setting }
            pEnumeration^.DefaultIndex := pE^.DefaultIndex; { default index setting }

            { assign base address of ItemList array to 'generic' pointer       }
            { i.e. reposition the struct pointer to overlay the allocated block}
            pVoid := @pEnumeration^.ItemList;
            for j := 0 to (pE^.NumItems - 1)
            do begin
               { dynamic cast to ItemType of each array element.               }
               case pE^.ItemType of
               TWTY_INT8   : TW_AINT8(pVoid^)[j]   := TW_AINT8(pList^)[j];
	       TWTY_INT16  : TW_AINT16(pVoid^)[j]  := TW_AINT16(pList^)[j];
    	       TWTY_INT32  : TW_AINT32(pVoid^)[j]  := TW_AINT32(pList^)[j];
    	       TWTY_UINT8  : TW_AUINT8(pVoid^)[j]  := TW_AUINT8(pList^)[j];
    	       TWTY_UINT16 : TW_AUINT16(pVoid^)[j] := TW_AUINT16(pList^)[j];
    	       TWTY_UINT32 : TW_AUINT32(pVoid^)[j] := TW_AUINT32(pList^)[j];
    	       TWTY_BOOL   : TW_ABOOL(pVoid^)[j]   := TW_ABOOL(pList^)[j];
	       TWTY_FIX32  : TW_AFIX32(pVoid^)[j]  := TW_AFIX32(pList^)[j];
               TWTY_FRAME  : TW_AFRAME(pVoid^)[j]  := TW_FRAME(TW_AFRAME(pList^)[j]);
	       TWTY_STR32  : lstrcpy(TW_ASTR32(pVoid^)[j],  TW_ASTR32(pList^)[j]);
	       TWTY_STR64  : lstrcpy(TW_ASTR64(pVoid^)[j],  TW_ASTR64(pList^)[j]);
	       TWTY_STR128 : lstrcpy(TW_ASTR128(pVoid^)[j], TW_ASTR128(pList^)[j]);
	       TWTY_STR255 : lstrcpy(TW_ASTR255(pVoid^)[j], TW_ASTR255(pList^)[j]);
	       else          begin { bad ItemType.                                  }
                             end;
               end;
            end;
            GlobalUnlock(pData^.hContainer);
       end;
  end;
  { now you have a dynamically sized enumeration array WITHIN a struct         }
end; { End BuildUpEnumerationType.                                             }


{------------------------------------------------------------------------------}
{ FUNCTION: BuildUpArrayType                                                   }
{                                                                              }
{ ARGS:  pData   pointer to a capability structure, details about container    }
{        pA      ptr to struct that contains the other fields of ARRAY struct  }
{        *pList  ptr to array of elements to put into the ARRAY struct         }
{                                                                              }
{ RETURNS: pData^.hContainer set to address of the container handle, ptr is    }
{          returned here                                                       }
{                                                                              }
{ NOTES: The routine dynamically allocates a chunk of memory large enough to   }
{ contain all the struct pTW_ARRAY as well as store it's ItemList array        }
{ INTERNAL to the struct.  The array itself and it's elements must be          }
{ type cast to ItemType.  I do not know how to dynamically cast elements       }
{ of an array to ItemType so it is time for a big static switch.>>>            }
{                                                                              }
{ Protocol: Used by MSG_GET.. calls were Source allocates the container and    }
{ the APP uses and then frees the container.                                   }
{------------------------------------------------------------------------------}

procedure BuildUpArrayType(pData : pTW_CAPABILITY;
                           pA    : pTW_ARRAY;
                           pList : pointer;
                           Msg   : TW_UINT16);
type TVoidList = array[0..0] of TW_UINT8;
var pArray : pTW_ARRAY;
    j      : integer;   { anyone with more than 32K array elements             }
                        { should crash.  Could type on NumItems.               }
{    pVoid  : ^TVoidList; }
    pVoid  : pointer;
begin
  if (Msg <> MSG_SET)
  then begin
       { allocate a block large enough for struct and complete array.          }
       pData^.hContainer := GlobalAlloc(GMEM_MOVEABLE,
		                        SizeOf(TW_ARRAY)  -
                                        SizeOf(TW_UINT8) +
		                        pA^.NumItems *
                                        AltTWItemSize(pA^.ItemType));
{       if (pData^.hContainer = 0)
       then Exit; }
  end;

  if (pData^.hContainer <> 0)
  then begin
       pData^.ConType := TWON_ARRAY;
       pArray := pTW_ARRAY(GlobalLock(pData^.hContainer));
       if (pArray <> Nil)
       then begin
            pArray^.ItemType := pA^.ItemType; { TWTY_XXXX                      }
            pArray^.NumItems := pA^.NumItems; { TWPT_XXXX...                   }

            { assign base address of ItemList array to 'generic' pointer       }
            { i.e. reposition the struct pointer to overlay the allocated block}
            pVoid := @pArray^.ItemList;

            for j := 0 to (pA^.NumItems - 1)
            do begin { dynamic cast to ItemType of each array element          }
               case pA^.ItemType of
	       TWTY_INT8   : TW_AINT8(pVoid^)[j]   := TW_AINT8(pList^)[j];
	       TWTY_INT16  : TW_AINT16(pVoid^)[j]  := TW_AINT16(pList^)[j];
    	       TWTY_INT32  : TW_AINT32(pVoid^)[j]  := TW_AINT32(pList^)[j];
    	       TWTY_UINT8  : TW_AUINT8(pVoid^)[j]  := TW_AUINT8(pList^)[j];
	       TWTY_UINT16 : TW_AUINT16(pVoid^)[j] := TW_AUINT16(pList^)[j];
	       TWTY_UINT32 : TW_AUINT32(pVoid^)[j] := TW_AUINT32(pList^)[j];
	       TWTY_BOOL   : TW_ABOOL(pVoid^)[j]   := TW_ABOOL(pList^)[j];
               TWTY_FIX32  : TW_AFIX32(pVoid^)[j]  := TW_AFIX32(pList^)[j];
	       TWTY_FRAME  : TW_AFRAME(pVoid^)[j]  := TW_AFRAME(pList^)[j];
	       TWTY_STR32  : lstrcpy(TW_ASTR32(pVoid^)[j],  TW_ASTR32(pList^)[j]);
	       TWTY_STR64  : lstrcpy(TW_ASTR64(pVoid^)[j],  TW_ASTR64(pList^)[j]);
	       TWTY_STR128 : lstrcpy(TW_ASTR128(pVoid^)[j], TW_ASTR128(pList^)[j]);
	       TWTY_STR255 : lstrcpy(TW_ASTR255(pVoid^)[j], TW_ASTR255(pList^)[j]);
	       else          begin { bad ItemType.                             }
                             end;
               end; { end of switch.                                           }
            end; { end of for loop.                                            }
            GlobalUnlock(pData^.hContainer);
       end;
  end;
  { now you have a dynamically sized array WITHIN a struct.                    }
end;  { End BuildUpArrayType.                                                  }


{------------------------------------------------------------------------------}
{ BuildUpRangeType -                                                           }
{------------------------------------------------------------------------------}

procedure BuildUpRangeType(pData : pTW_CAPABILITY;
                           pR    : pTW_RANGE;
                           Msg   : TW_UINT16);
var pRange : pTW_RANGE;
begin
  if (msg <> MSG_SET) { MSG_SET allocates the structure at the app.            }
  then begin
       pData^.hContainer := GlobalAlloc(GMEM_MOVEABLE,
	    	                        SizeOf(TW_RANGE));
  end;
  if (pData^.hContainer <> 0)
  then begin
       pData^.ConType := TWON_RANGE;
       pRange := pTW_RANGE(GlobalLock(pData^.hContainer));
       if (pRange <> Nil)
       then begin
            pRange^.ItemType     := pR^.ItemType;
	    pRange^.MinValue     := pR^.MinValue;
	    pRange^.MaxValue     := pR^.MaxValue;
	    pRange^.StepSize     := pR^.StepSize;
	    pRange^.DefaultValue := pR^.DefaultValue;
	    pRange^.CurrentValue := pR^.CurrentValue;

	    GlobalUnlock(pData^.hContainer);
       end;
  end;
end; { End BuildUpRangeType.                                                   }


{------------------------------------------------------------------------------}
{ FUNCTION: BuildUpOneValue                                                    }
{                                                                              }
{ ARGS:    pData   pointer to a capability structure, details about container  }
{          ItemType constant that defines the type of the Item to follow       }
{          Item    the data to put into the OneValue container                 }
{                                                                              }
{ RETURNS: pData^.hContainer set to address of the container handle, ptr is    }
{          returned here                                                       }
{                                                                              }
{ NOTES:   This routine responds to a CAP_ call by creating a container of     }
{ type OneValue and returning with the hContainer value (excuse me) "pointing" }
{ to the container.  The container is filled with the values for ItemType      }
{ and Item requested by the caller.                                            }
{                                                                              }
{ NOTE: be sure to tell the APP the container type you have built, ConType.    }
{                                                                              }
{ Protocol: Used by MSG_GET.. calls were Source allocates the container and    }
{ the APP uses and then frees the container.                                   }
{                                                                              }
{ For generalization value of ItemType does not affect Item in anyway.         }
{ Caller should cast Item to TW_UINT32.                                        }
{------------------------------------------------------------------------------}

function BuildUpOneValue(pData    : pTW_CAPABILITY;
                         ItemType : TW_UINT16;
                         Item     : pointer;
                         Msg      : TW_UINT16) : TW_UINT16;
var pOneValue : pTW_ONEVALUE;
  {$IFDEF VER70}
    Result : TW_UINT16;
  {$ENDIF}
begin
  Result := TWCC_SUCCESS;
  if (Msg <> MSG_SET) { MSG_SET allocates the structure at the app.            }
  then begin
       pData^.hContainer := GlobalAlloc(GMEM_MOVEABLE,
	    	                        SizeOf(TW_ONEVALUE));
       if (pData^.hContainer = 0)
       then Result := TWCC_LOWMEMORY;
  end;

  if (pData^.hContainer <> 0)
  then begin { tell APP the ConType returning.                                 }
       pData^.ConType := TWON_ONEVALUE;
       pOneValue := pTW_ONEVALUE(GlobalLock(pData^.hContainer));
       if (pOneValue <> Nil)
       then begin
            pOneValue^.ItemType := ItemType;
            pOneValue^.Item     := 0; // The
            case ItemType of
            TWTY_INT8   : pOneValue^.Item := TW_INT8(Item^);
	    TWTY_INT16  : pOneValue^.Item := TW_INT16(Item^);
    	    TWTY_INT32  : pOneValue^.Item := TW_INT32(Item^);
    	    TWTY_UINT8  : pOneValue^.Item := TW_UINT8(Item^);
    	    TWTY_UINT16 : pOneValue^.Item := TW_UINT16(Item^);
    	    TWTY_UINT32 : pOneValue^.Item := TW_UINT32(Item^);
    	    TWTY_BOOL   : pOneValue^.Item := TW_BOOL(Item^);
	    TWTY_FIX32 : begin
			   { fix32.Whole := @Item^.Whole;                      }
			   { fix32.Frac  := 32;                                }
			   { pOneValue^.Item := pTW_INT32(@fix32);             }
			   { pOneValue^.Item := pTW_INT32(@Item);              }
	                   pOneValue^.Item := TW_UINT32(Item^); //Longint(Item^)
			 end;

//          TWTY_FRAME  : pOneValue^.Item  := TW_FRAME(TW_AFRAME(pList^)[j]);
//          TWTY_STR32  : lstrcpy(TW_ASTR32(pVoid^)[j],  TW_ASTR32(pList^)[j]);
//          TWTY_STR64  : lstrcpy(TW_ASTR64(pVoid^)[j],  TW_ASTR64(pList^)[j]);
//          TWTY_STR128 : lstrcpy(TW_ASTR128(pVoid^)[j], TW_ASTR128(pList^)[j]);
//	    TWTY_STR255 : lstrcpy(TW_ASTR255(pVoid^)[j], TW_ASTR255(pList^)[j]);
            else         begin
	                   pOneValue^.Item := TW_UINT32(Item^);
                         end;
            end;
       end;
       GlobalUnlock(pData^.hContainer);
       Result := TWCC_SUCCESS;
  end;
  BuildUpOneValue := Result;
end; { End BuildUpOneValue.                                                    }


{------------------------------------------------------------------------------}
{ FUNCTION: ExtractOneValue                                                    }
{                                                                              }
{ ARGS:    pData   pointer to a capability structure, details about container  }
{          pVoid   ptr will be set to point to the item on return              }
{                                                                              }
{ RETURNS: pVoid pts to extracted value.                                       }
{                                                                              }
{ NOTES:   This routine will open a container and extract the Item.  The Item  }
{ will be returned to the caller in pVoid.  I will type cast the returned      }
{ value to that of ItemType.                                                   }
{                                                                              }
{ Protocol: used by MSG_SET calls were Source empties then App frees.  It is   }
{ also assumed that the APP allocates and fills the container BEFORE this      }
{ call.                                                                        }
{------------------------------------------------------------------------------}

procedure ExtractOneValue(pData : pTW_CAPABILITY;
                          pVoid : pointer);
var pOneValue : pTW_ONEVALUE;
begin
  pOneValue := pTW_ONEVALUE(GlobalLock(pData^.hContainer));
  if (pOneValue <> Nil)
  then begin
       { add a check for valid type.                                           }
       { CAST to type of var caller wants.                                     }
       case pOneValue^.ItemType of
       TWTY_INT8   : TW_INT8(pVoid^)   := TW_INT8(pOneValue^.Item);
       TWTY_UINT8  : TW_UINT8(pVoid^)  := TW_UINT8(pOneValue^.Item);
       TWTY_INT16  : TW_INT16(pVoid^)  := TW_INT16(pOneValue^.Item);
       TWTY_UINT16 : TW_UINT16(pVoid^) := TW_UINT16(pOneValue^.Item);
       TWTY_INT32  : TW_INT32(pVoid^)  := TW_INT32(pOneValue^.Item);
       TWTY_UINT32 : TW_UINT32(pVoid^) := TW_UINT32(pOneValue^.Item);
       TWTY_BOOL   : TW_BOOL(pVoid^)   := TW_BOOL(pOneValue^.Item);
(*
       TWTY_INT8   : pTW_INT8(pVoid)^   := TW_INT8(pOneValue^.Item);
       TWTY_UINT8  : pTW_UINT8(pVoid)^  := TW_UINT8(pOneValue^.Item);
       TWTY_INT16  : pTW_INT16(pVoid)^  := TW_INT16(pOneValue^.Item);
       TWTY_UINT16 : pTW_UINT16(pVoid)^ := TW_UINT16(pOneValue^.Item);
       TWTY_INT32  : pTW_INT32(pVoid)^  := TW_INT32(pOneValue^.Item);
       TWTY_UINT32 : pTW_UINT32(pVoid)^ := TW_UINT32(pOneValue^.Item);
       TWTY_BOOL   : pTW_BOOL(pVoid)^   := TW_BOOL(pOneValue^.Item);
*)
       TWTY_FIX32  : pTW_FIX32(pVoid)^  := TW_FIX32(pOneValue^.Item);
       {$IFDEF DEBUG}
{ Bugger
       TWTY_FRAME  : pTW_FRAME(pVoid)^  := TW_FRAME(pOneValue^.Item);
       TWTY_STR32  : pTW_STR32(pVoid)^  := TW_STR32(pOneValue^.Item);
       TWTY_STR64  : pTW_STR64(pVoid)^  := TW_STR64(pOneValue^.Item);
       TWTY_STR128 : pTW_STR128(pVoid)^ := TW_STR128(pOneValue^.Item);
       TWTY_STR255 : pTW_STR255(pVoid)^ := TW_STR255(pOneValue^.Item);
}
       {$ENDIF}
       else          begin
                     end;
       end;
       GlobalUnlock(pData^.hContainer);
  end;
  { it is assumed that the App will free the container upon return.            }
end; { End ExtractOneValue.                                                    }



{------------------------------------------------------------------------------}
{ FUNCTION: AltTWItemSize                                                      }
{                                                                              }
{ ARGS:    ItemType    constant which serves as an index to appropiate         }
{                      data type                                               }
{ RETURNS: Result      size of item in bytes                                   }
{                                                                              }
{ NOTES:   The routine provides a look-up table to get actual size in bytes of }
{ a particular data type.  Using the SizeOf call should give the correct       }
{ results on any machine type.  The value for the ItemType parm are found      }
{ in the twain.h file and are indicated by TWTY_XXXX.. (TWTY_UINT16...)        }
{                                                                              }
{ This routine is provided to allow the Source to use this code intact.  The   }
{ Source does not link to the TWA_GLUE.C module and thus does not have access  }
{ to the TWItemSize array.                                                     }
{------------------------------------------------------------------------------}

function AltTWItemSize(ItemType : TW_INT16) : TW_UINT16;
{$IFDEF VER70}
var Result : TW_UINT16;
{$ENDIF}
begin
  Result := SizeOf(TW_UINT16);
  case ItemType of
  TWTY_INT8   : Result := SizeOf(TW_INT8);
  TWTY_UINT8  : Result := SizeOf(TW_UINT8);
  TWTY_INT16  : Result := SizeOf(TW_INT16);
  TWTY_UINT16 : Result := SizeOf(TW_UINT16);
  TWTY_INT32  : Result := SizeOf(TW_INT32);
  TWTY_UINT32 : Result := SizeOf(TW_UINT32);
  else          begin
                end;
  end;
  AltTWItemSize := Result;
end; { End AltTWItemSize.                                                      }


end. { End Unit twd_com.                                                       }
